//#pragma warning(disable: 4819)
//#pragma comment(linker, "/subsystem:\"windows\" /entry:\"mainCRTStartup\"")


#include <windows.h>
#include <stdio.h>
#include "cv.h"
#include "cxcore.h"
#include "highgui.h"
#include "util.h"

#define PIXVAL(iplimagep, x, y) (*(uchar *)((iplimagep)->imageData + (y) * (iplimagep)->widthStep + (x)))

#define QPC(cnloop, i) QueryPerformanceCounter(&(cnloop)[i])
#define LOOP_MAX 100
#define CN_MAX 10

LARGE_INTEGER freq;
LARGE_INTEGER cn[LOOP_MAX][CN_MAX];


int main()
{
	int i, j;
	int loop = 0;
	FILE *fp;

	CvCapture *capture = cvCaptureFromCAM(0);

	IplImage *frame, *buf, *img, *back, *diff;

	if (!capture) {
		fprintf(stderr, "cannot capture\n");
		return 1;
	}

	frame = cvQueryFrame(capture);
	buf = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 3);  // color
	img = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
	back = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);
	diff = cvCreateImage(cvGetSize(frame), IPL_DEPTH_8U, 1);

	cvNamedWindow("image", CV_WINDOW_AUTOSIZE);
	cvNamedWindow("frame", CV_WINDOW_AUTOSIZE);

	if (QueryPerformanceFrequency(&freq) == 0) {
		fprintf(stderr, "cannot use performance counter\n");
		return 1;
	}

	while (1) {
		QPC(cn[loop], 0);
		frame = cvQueryFrame(capture); /* cannot be modified */
		QPC(cn[loop], 1);

		cvFlip(frame, buf, NULL);

		QPC(cn[loop], 2);
		cvCvtColor(buf, img, CV_BGR2GRAY);

		QPC(cn[loop], 3);

		for (j = 0; j < img->height; j++) {
			for (i = 0; i < img->width; i++) {
				int d = PIXVAL(img, i, j) - PIXVAL(back, i, j);
				PIXVAL(diff, i, j) = d/2 + 128;
			}
		}
		QPC(cn[loop], 4);
		cvCopy(img, back, 0);
		QPC(cn[loop], 5);

		cvShowImage("image", diff);
		cvShowImage("frame", buf);
		QPC(cn[loop], 6);

		if (cvWaitKey(30) > 0) {
			break;
		}

		QPC(cn[loop], 7);

		loop = (loop + 1) % LOOP_MAX;
	}

	printf("freq = %f [Hz]\n", (double)(freq.QuadPart));
	for (i = 1; i < 10; i++) {
		printf("%02i-%02i: %f [us]\n", i, i-1, 
			(1000000.0 * (cn[loop][i].QuadPart - cn[loop][i-1].QuadPart)) / freq.QuadPart);
	}

	fp = fopen("./time.dat", "w");
	if (fp == NULL) {
		fprintf(stderr, "can't open time.dat");
		return 1;
	}
	for (j = 0; j < LOOP_MAX; j++) {
		for (i = 1; i < CN_MAX; i++) {
			double t = (1000000.0 *
				(cn[(loop + j) % LOOP_MAX][i].QuadPart 
				- cn[(loop + j) % LOOP_MAX][i-1].QuadPart)) / freq.QuadPart;
			fprintf(fp, "%d %f\n", i, t >= 0.0? t: 0.0);
		}
	}
	fclose(fp);

	return 0;
}
